#!/bin/sh

## HELP AND BASIC ARGUMENT PROCESSING
#####################################

PROG="${0}"
EXTRAVER=""
ISODIR=/worksrc/isofiles
MOUNTDIR=/mnt/cdrom
WORKDIR=/worksrc/catalyst/isotemp
DESTDIR=/home/sysresccdiso
REPOSRC=/worksrc/sysresccd-src
REPOBIN=/worksrc/sysresccd-bin
MYDATE=$(date +%Y%m%d-%Hh%M)

usage()
{
	cat <<EOF
Usage: ${PROG} [options] <arch>

Rebuild the distribution iso by applying the SystemRescueCD overlays
and recent kernels to a previous vesion.

arch: x86 | amd64 | sparc

Options:
 -h|--help         Display this screen and exit
 -i ISODIR         Directory containing the iso file to rebuild ($ISODIR)
                     <isodir>/systemrescuecd-<arch>-current.iso
 -w WORKDIR        Set the work directory for the custom CD (${WORKDIR})
 -r REPOSRC        Location of git systemrescuecd repo ($REPOSRC)
 -b REPOBIN        ? ($REPOBIN)
 -d DESTDIR        Set the directory for saving the rebuild iso file ($DESTDIR)
 -m MOUNTDIR       Set the directory for mounting the current iso ($MOUNTDIR)
 -e EXTRAVER       Extra version information for f1boot.msg ($EXTRAVER)

Distributed under the GNU Public License version 2 - http://www.sysresccd.org
EOF
}

while getopts ":i:w:r:b:d:m:e:" Option
do
	case $Option in
		i ) ISODIR="$OPTARG";;
		w ) WORKDIR="$OPTARG";;
		r ) REPOSRC="$OPTARG";;
		b ) REPOBIN="$OPTARG";;
		d ) DESTDIR="$OPTARG";;
		m ) MOUNTDIR="$OPTARG";;
		e ) EXTRAVER="$OPTARG";;
		* ) usage; exit 1;;  # Default, handles -h
	esac
done
shift $(($OPTIND - 1))

ARCH="$1"
if [ "$1" = "x86" ] || [ "$1" = "amd64" ] || [ "$1" = "sparc" ]
then
	ARCH="$1"
else
fi
case ${ARCH} in
	x86)    BASEARCH="x86";;
	amd64)  BASEARCH="x86";;
	sparc)  BASEARCH="sparc";;
	*)      usage; exit 1;;
esac
VERSION=$(cat ${REPOSRC}/overlay-squashfs-${BASEARCH}/root/version)
VOLNAME="sysrcd-${VERSION}" 

## ERROR HANDLING
#####################################

die()
{
	if [ -n "$1" ]
	then
		echo "${PROG}: error: $1"
	else
		echo "${PROG}: aborting."
	fi
	exit 1
}

# ========= copy files from the temp iso image ================================
CURFILE="${ISODIR}/systemrescuecd-${ARCH}-current.iso"

mkdir -p ${DESTDIR}

[ -f "${CURFILE}"  ] || die "Cannot find \"${CURFILE}\""

umount ${MOUNTDIR} 2>/dev/null || die "Cannot unmount \"${MOUNTDIR}\""
mount -o loop,ro ${CURFILE} ${MOUNTDIR} || die "Cannot mount \"${CURFILE}\""

[ -f ${MOUNTDIR}/image.squashfs ] || die "Cannot find a valid file in the ISO"

[ -d ${WORKDIR} ] && rm -rf ${WORKDIR}
mkdir -p ${WORKDIR}
cp ${MOUNTDIR}/isolinux/rescuecd* ${REPOBIN}/kernels-${ARCH}/
cp ${MOUNTDIR}/image.squashfs ${WORKDIR}/sysrcd.dat
( cd ${WORKDIR} ; md5sum sysrcd.dat > sysrcd.md5 ; chmod 644 sysrcd.* ) 
umount ${MOUNTDIR}

# ========= copy files from overlays ===========================================
rsync -ax ${REPOBIN}/overlay-iso-${BASEARCH}/ "${WORKDIR}/"
rsync -ax ${REPOSRC}/overlay-iso-${BASEARCH}/isolinux/ "${WORKDIR}/isolinux/"
rsync -ax ${REPOBIN}/kernels-${ARCH}/ ${WORKDIR}/isolinux/ --exclude='*.igz'
cp ${REPOSRC}/overlay-squashfs-${BASEARCH}/root/version ${WORKDIR}

# ========= integrate the version number in f1boot.msg =========================
sed -i -e "s/VERSION/${VERSION}${EXTRAVER}/" ${WORKDIR}/isolinux/f1boot.msg

# ========= merge (rescuecd.igz+rescue64.igz+altker32.igz) --> rescuecd.igz ====
curdir="${WORKDIR}/isolinux"
newramfs="${curdir}/initram-root"
newinitrfs="${curdir}/initram.igz"
modulesdir="${newramfs}/lib/modules"

# prepare root of new initramfs
[ -d ${newramfs} ] && rm -rf ${newramfs}
mkdir -p ${newramfs}
cp -a ${REPOBIN}/overlay-initramfs/* ${newramfs}/
mkdir -p ${modulesdir}

# extract the old ramdisks
for ker in rescuecd rescue64 altker32 altker64
do
        oldimg="${REPOBIN}/kernels-${ARCH}/${ker}.igz"
        newdir="${curdir}/${ker}-tmp"
        echo "extracting ${oldimg}..."
        mkdir -p "${newdir}"
        ( cd "${newdir}" && cat ${oldimg} | gzip -d | cpio -id 2>/dev/null )
done

# copy {rescue64,altker32,altker64}/lib/modules to the new initramfs
for ker in rescuecd rescue64 altker32 altker64
do
        cp -a ${curdir}/${ker}-tmp/lib/modules/* ${modulesdir}/
done

# copy custom busybox binary to the new initramfs
cp ${curdir}/rescuecd-tmp/bin/busybox ${newramfs}/bin/
( cd ${newramfs}/bin/ ; ln busybox sh )

# update the init boot script in the initramfs
cp ${REPOSRC}/mainfiles/linuxrc* ${newramfs}/
cp ${REPOSRC}/mainfiles/linuxrc ${newramfs}/init

# strip and compress kernel modules which are in the sysrcd.dat to save space
find ${modulesdir} -name "*.ko" -exec strip --strip-unneeded '{}' \;
find ${modulesdir} -name "*.ko" -exec gzip '{}' \;

# build new initramfs
echo 'building the new initramfs...'
( cd ${newramfs} && find . | cpio -H newc -o | gzip -9 > ${newinitrfs} )

# remove old igz-images and tmp-dirs
[ -d ${newramfs} ] && rm -rf ${newramfs} 
for ker in rescuecd rescue64 altker32 altker64
do
        #[ -f "${curdir}/${ker}.igz" ] && rm -f "${curdir}/${ker}.igz"
        [ -d "${curdir}/${ker}-tmp" ] && rm -rf "${curdir}/${ker}-tmp"
done

# ========= copy the new files to the pxe environment =========================
if [ -d /tftpboot ]
then
	cp ${WORKDIR}/sysrcd.dat /tftpboot/
	cp ${WORKDIR}/sysrcd.md5 /tftpboot/
	cp ${WORKDIR}/isolinux/* /tftpboot/
fi

# ========= prepare the ISO image =============================================
ISOFILE="${DESTDIR}/systemrescuecd-${ARCH}-${VERSION}-${MYDATE}.iso"

if [ "${ARCH}" = "x86" ] || [ "${ARCH}" = "amd64" ]
then
	mkisofs -J -l -V ${VOLNAME} -input-charset utf-8 -o ${ISOFILE} -b isolinux/isolinux.bin -c isolinux/boot.cat -no-emul-boot -boot-load-size 4 -boot-info-table ${WORKDIR}
fi

if [ "${ARCH}" = "sparc" ]
then
	mkisofs -G /boot/isofs.b -J -V ${VOLNAME} -B ... -r -o ${ISOFILE} ${WORKDIR}
fi

# ========= prepare the backup ==================================================
tar cfjp "${DESTDIR}/systemrescuecd-${ARCH}-${VERSION}-${MYDATE}.tar.bz2" ${REPOSRC} ${REPOBIN} --exclude='.git'
